import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';

export default class PlotComponent extends Component {
  @tracked dimensions;
  @tracked elementWidth;
  @tracked elementHeight;
  @tracked yRange;

  get heightRatio() {
    return this.args.heightRatio || 0.5;
  }

  get height() {
    return (this.elementWidth ? this.elementWidth : 0) * this.heightRatio;
  }

  get useElementHeight() {
    return this.args.useElementHeight || false;
  }

  get transform() {
    return this.args.transform || "translate(0,0)";
  }

  get hasData() {
    return this.args.data && this.args.data.length > 0;
  }

  get isMultiDataset() {
    return this.hasData && this.args.data[0] && this.args.data[0].constructor === Array;
  }

  get isMultiRange() {
    return this.args.timeRange && this.args.timeRange[0].constructor === Array;
  }

  get matchMultiRangeXscale() {
    return this.args.matchMultiRangeXscale || true;
  }

  get scaleYBasedOnTotalDataset() {
    return this.args.scaleYBasedOnTotalDataset;
  }

  get showingYLabel() {
    return !this.args.isLoading && this.args.yLabel;
  }

  get showingXLabel() {
    return !this.args.isLoading && this.args.xLabel;
  }

  get showXTicks() {
    return this.args.showXTicks || false;
  }

  get xTickUnits() {
    return this.args.xTickUnits || 'date';
  }

  get showYTicks() {
    return this.args.showYTicks || false;
  }

  get yTickUnits() {
    return this.args.yTickUnits || 'float';
  }

  get timeInterval() {
    return this.args.timeInterval || 1000*60*60*24 // default daily - used to calculate and place x ticks
  }

  get hasFilter() {
    return this.args.hasFilter;
  }

  sortDates(d1, d2) {
    return d1 - d2;
  }

  get groupDimensions() {
    return {
      height: this.dimensions ? this.dimensions.svg.clientHeight : 0,
      width: this.dimensions ? this.dimensions.svg.clientWidth : 0
    }
  }

  get sortedTimeRange() {
    return this.args.timeRange
      ? this.isMultiRange
        ? this.adjustRanges(this.args.timeRange.map(range => this.matchTimeInterval(range.sort(this.sortDates))))
        : this.matchTimeInterval(this.args.timeRange.sort(this.sortDates))
      : [new Date(), new Date()]
  }

  // This is a method used to adjust ranges to match time interval
  // on a bar graph, if they are looking at daily data, and they choose last 7 days from now (assuming its midday)
  // The first bar will be cut off a little bit becuase it will start from current time, this rounds it to timeInterval,
  // instead of starting on 3/24/2021 10:30am, rounds it to 3/24/2021 12:00am for daily
  matchTimeInterval(range) { 
    if(!this.timeInterval) return range;
    return range.map((date) => {
      return new Date(
        date.getFullYear(),
        date.getMonth(),
        date.getDate(),
        this.timeInterval >= 1000*60*60*24 ? 0 : date.getHours(), // daily or greater interval start at midnight
        this.timeInterval >= 1000*60*60 ? 0 : date.getMinutes(), // hourly or greater interval start on the hour
        this.timeInterval >= 1000*60 ? 0 : date.getSeconds() // hourly or greater interval start on the hour
      )
    })
  }

  adjustRanges(ranges) {
    if(ranges && this.isMultiRange && this.matchMultiRangeXscale) {
      const rangeLengths = ranges.map(range => range[1] - range[0])
      const maxRange = Math.max(...rangeLengths)
      return ranges.map((range) => {
        const additionalScale = maxRange - (range[1] - range[0])
        return [ range[0], new Date(range[1].getTime() + additionalScale) ]
      })
    } else {
      return ranges;
    }
  }

  get xKey() {
    return this.args.xKey ? this.args.xKey : 'at';
  }

  get yKey() {
    return this.args.yKey ? this.args.yKey : 'value';
  }

  get y0() {
    return this.args.manualY0 || 0;
  }

  get filteredData() {
    if (!this.hasData) return []
    if (this.isMultiDataset && this.isMultiRange) {
      return this.sortedTimeRange.map((range, index) => {
        return this.filterData(this.args.data[index] || [], range)
      })
    } else if (this.isMultiDataset) {
      return this.args.data.map(dataset => this.filterData(dataset));
    } else if (this.isMultiRange) {
      return this.sortedTimeRange.map(range => this.filterData(this.args.data, range))
    } else {
      return this.filterData(this.args.data);
    }
  }

  filterData(dataset, timeRange) {
    if(!this.hasFilter) return dataset
    const range = timeRange ? timeRange : this.sortedTimeRange;
    return dataset.filter((dataPoint) => {
      return new Date(dataPoint[this.xKey]).getTime() >= range[0].getTime() && new Date(dataPoint[this.xKey]).getTime() <= range[1].getTime();
    })
  }

  // Amount of space to buffer the min data (if set to zero, lowest datapoint will be on x axis)
  get dataBottomPadding() {
    return this.args.dataBottomPadding ? this.args.dataBottomPadding : 0
  }

  resize = () => {
    const chart = this.dimensions.svg;
    this.elementWidth = chart.getBoundingClientRect().width;
    this.dimensions = { width: this.elementWidth, height: this.height, svg: chart };
    if(this.args.updateDimensions) this.args.updateDimensions(this.dimensions);
  }

  initializeChart = (el) => {
    const boundingRect = el.getBoundingClientRect() 
    this.elementWidth = boundingRect.width;
    this.elementHeight = boundingRect.height;
    this.dimensions = {width: this.elementWidth, height: this.useElementHeight ? this.elementHeight : this.height, svg: el};
    if(this.args.updateDimensions) this.args.updateDimensions( this.dimensions );
  }

  pad(number, padding = -2) {
    return ('0' + number).slice(padding)
  }
  
  formattedTime(date) {
    const hours = date.getHours()
    const nonMilitary = hours > 12 ? hours - 12 : hours;
    const amPm = hours >= 12 ? 'pm' : 'am';
    return `${nonMilitary === 0 ? 12 : nonMilitary}:${this.pad(date.getMinutes())} ${amPm}`;
  }
  
  formatDate(date, hideTime) {
    if(!date || typeof date !== 'object' || !date.getMonth) return ""
    const time = hideTime ? '' : this.formattedTime(date);
    return `${date.getMonth() + 1}-${date.getDate()}-${date.getFullYear()% 100} ${time}`;
  }
  
} 